package com.winhc.bigdata.spark.model

import java.util.Date

import com.winhc.bigdata.calc.DimScoreV2
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils, SparkUtils}
import org.apache.spark.sql.{Row, SparkSession}

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * 知识产权类得分
 * 软件著作权 | 作品著作权 | 专利 | 商标 | 网站
 * π
 */

object CompanyIntellectualsScore {

  val tabMapping: Map[String, (String, String, String, String)] =
    Map("ads_company_copyright_reg_list" -> ("1", "publish_time", "资产权益", "软著作权"), //软件著作权
      "ads_company_copyright_works_list" -> ("2", "finish_time", "资产权益", "著作权"), //作品著作权
      "ads_company_patent_list" -> ("3;4", "pub_date", "资产权益", "实用新型、外观设计专利;发明专利"), //专利
      "ads_company_icp" -> ("5", "examine_date", "资产权益", "网站"), //网站
      "ads_company_tm" -> ("6", "app_date", "资产权益", "商标"), //商标
      "ads_company_land_announcement" -> ("7", "commit_time", "资产权益", "购地信息") //购地信息
    )

  def main(args: Array[String]): Unit = {

    val (sourceTable, flag, time, kind, project) = valid(args)

    var config = mutable.Map(
      "spark.hadoop.odps.project.name" -> "winhc_eci_dev",
      "spark.hadoop.odps.spark.local.partition.amt" -> "10"
    )
    config.+=("spark.hadoop.odps.project.name" -> "winhc_eci_dev")
    println(config)
    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)

    println(s"company ${this.getClass.getSimpleName} calc start! " + new Date().toString)

    //专利分成两部分
    if (flag.contains(";")) {
      flag.split(";").foreach(f => {
        new CompanyIntellectualsScore(spark, sourceTable, f, time, kind, project).calc()
      })
    } else {
      new CompanyIntellectualsScore(spark, sourceTable, flag, time, kind, project).calc()
    }

    println(s"company ${this.getClass.getSimpleName} calc end! " + new Date().toString)
    spark.stop()

  }

  def valid(args: Array[String]) = {
    if (args.length != 1) {
      println("请输入要计算的table!!!! ")
      sys.exit(-1)
    }
    val Array(sourceTable) = args
    println(sourceTable.substring(sourceTable.indexOf(".") + 1))

    val (flag, time, kind, project) = tabMapping.getOrElse(sourceTable.substring(sourceTable.indexOf(".") + 1), ("", "", "", ""))
    if (flag.isEmpty || time.isEmpty || kind.isEmpty || project.isEmpty) {
      println("输入表不存在!!!   ")
      sys.exit(-1)
    }
    (sourceTable, flag, time, kind, project)
  }
}

case class CompanyIntellectualsScore(s: SparkSession, sourceTable: String,
                                     flag: String, time: String, kind: String, project: String
                                    ) extends LoggingUtils {

  @(transient@getter) val spark: SparkSession = s

  import spark.implicits._

  def calc(): Unit = {
    //val targetTable = "ads_company_total_score"
    val targetTable = sourceTable + "_score"
    val ds = BaseUtil.getPartion(sourceTable, spark)

    var appsql = ""
    var apptab = ""

    // 专利类型(1（A）.发明公布，2（B,C）.发明授权，3（U，Y）.实用新型，4（D，S）.外观设计)
    if ("3".equals(flag)) { //实用新型、外观设计专利
      appsql = "AND SUBSTR(pub_number, -1) in ('U','Y','D','S')"
      apptab = "_s"
    } else if ("4".equals(flag)) { //发明专利
      appsql = "AND SUBSTR(pub_number, -1) in ('A','B','C')"
      apptab = "_f"
    }

    val df = sql(
      s"""
         |SELECT  *
         |FROM    (
         |        SELECT
         |                *
         |                ,COUNT(new_cid) OVER(PARTITION BY new_cid ) AS cnt1
         |                ,row_number() OVER(PARTITION BY new_cid ORDER BY $time DESC ) AS num
         |        FROM    $sourceTable
         |        WHERE   ds = ${ds}
         |        AND     new_cid IS NOT NULL ${appsql}
         |        ) a
         |WHERE   num =1
         |""".stripMargin)

    df.map(r => {
      trans(r, flag, kind, project)
    }).toDF("id", "cid", "kind", "kind_code", "project", "project_code", "type",
      "score", "total", "extraScore")
      .createOrReplaceTempView(s"t1_view")

    //    logger.info(
    //      s"""
    //         |- - - - - - - - - - - - - - - - - - - - - - - - -
    //         |${showString(sql(s"select * from t1_view"))}
    //         |- - - - - - - - - - - - - - - - - - - - - - - - -
    //           """.stripMargin)

    sql(s"insert overwrite table ${targetTable}${apptab} " +
      s"partition (ds='${ds}')  select * from t1_view")
  }

  def trans(r: Row, flag: String, kind: String, prpject: String) = {
    val id = r.getAs[Long]("id")
    val cid = r.getAs[Long]("new_cid").toString
    val cnt1 = r.getAs[Long]("cnt1")
    flag match {
      case "1" => softCopyRightScore(id, cid, cnt1, kind, prpject)
      case "2" => copyRightScore(id, cid, cnt1, kind, prpject)
      case "3" => ordinaryPatentScore(id, cid, cnt1, kind, prpject)
      case "4" => inventScore(id, cid, cnt1, kind, prpject)
      case "5" => webSiteScore(id, cid, cnt1, kind, prpject)
      case "6" => tradeMarkScore(id, cid, cnt1, kind, prpject)
      case "7" => immovableScore(id, cid, cnt1, kind, prpject)
    }
  }

  //软著作权
  def softCopyRightScore(id: Long, cid: String, cnt1: Long, kind: String, project: String) = {
    var score = 0f
    val total = 3f
    val extraScore = 0f
    var ty = ""
    if (cnt1 > 50) {
      score = 8f
      ty = "软件著作权数量＞50，该项分数上加5分"
    } else if (cnt1 > 5) {
      score = 3f
      ty = "授予数量＞5"
    } else if (cnt1 > 0) {
      score = 2f
      ty = "授予数量≤5"
    } else {
      score = 1f
      ty = "无"
    }
    (id, cid, kind, DimScoreV2.newsEventMap.get(kind), project, DimScoreV2.newsEventMap.get(project), ty,
      score, total, extraScore)
  }

  //著作权
  def copyRightScore(id: Long, cid: String, cnt1: Long, kind: String, project: String) = {
    var score = 0f
    val total = 2f
    val extraScore = 0f
    var ty = ""
    if (cnt1 > 3) {
      score = 2f
      ty = "授予数量＞3"
    } else if (cnt1 == 0) {
      score = 0f
      ty = "无"
    } else {
      score = 1f
      ty = "授予数量≤3"
    }
    (id, cid, kind, DimScoreV2.newsEventMap.get(kind), project, DimScoreV2.newsEventMap.get(project), ty,
      score, total, extraScore)
  }

  //普通专利
  def ordinaryPatentScore(id: Long, cid: String, cnt1: Long, kind: String, project: String) = {
    val project_s = project.split(";")(0)
    var score = 0f
    val total = 3f
    val extraScore = 0f
    var ty = ""
    if (cnt1 == 0) {
      score = 1f
      ty = "无"
    } else if (cnt1 <= 5) {
      score = 2f
      ty = "授予数量≤5"
    } else {
      score = 3f
      ty = "授予数量＞5"
    }
    (id, cid, kind, DimScoreV2.newsEventMap.get(kind), project_s, DimScoreV2.newsEventMap.get(project_s), ty,
      score, total, extraScore)
  }

  //发明专利
  def inventScore(id: Long, cid: String, cnt1: Long, kind: String, project: String) = {
    val project_s = project.split(";")(1)
    var score = 0f
    val total = 7f
    val extraScore = 0f
    var ty = ""
    if (cnt1 == 0) {
      score = 3f
      ty = "无"
    } else if (cnt1 <= 3) {
      score = 5f
      ty = "授予数量≤3"
    } else if (cnt1 <= 50) {
      score = 7f
      ty = "授予数量＞3"
    } else {
      score = 12f
      ty = "发明专利数量＞50，该项分数上加5分"
    }
    (id, cid, kind, DimScoreV2.newsEventMap.get(kind), project_s, DimScoreV2.newsEventMap.get(project_s), ty,
      score, total, extraScore)
  }

  //网站
  def webSiteScore(id: Long, cid: String, cnt1: Long, kind: String, project: String) = {
    var score = 0f
    val total = 1f
    val extraScore = 0f
    var ty = ""
    if (cnt1 > 20) {
      score = 4f
      ty = "网站域名数量＞20，该项分数上加3分"
    } else if (cnt1 > 0) {
      score = 1f
      ty = "有"
    } else {
      score = 0f
      ty = "无"
    }
    (id, cid, kind, DimScoreV2.newsEventMap.get(kind), project, DimScoreV2.newsEventMap.get(project), ty,
      score, total, extraScore)
  }

  //商标
  def tradeMarkScore(id: Long, cid: String, cnt1: Long, kind: String, project: String) = {
    var score = 0f
    val total = 4f
    val extraScore = 0f
    var ty = ""
    if (cnt1 == 0) {
      score = 0f
      ty = "无"
    } else if (cnt1 <= 2) {
      score = 2f
      ty = "授予数量≤2"
    } else if (cnt1 <= 8) {
      score = 3f
      ty = "授予数量＞2，≤8"
    } else if (cnt1 <= 100) {
      score = 4f
      ty = "授予数量＞8"
    } else {
      score = 9f
      ty = "商标数量＞100，该项分数上加5分"
    }
    (id, cid, kind, DimScoreV2.newsEventMap.get(kind), project, DimScoreV2.newsEventMap.get(project), ty,
      score, total, extraScore)
  }

  //购地信息
  def immovableScore(id: Long, cid: String, cnt1: Long, kind: String, project: String) = {
    var score = 0f
    val total = 15f
    val extraScore = 0f
    var ty = ""
    if (cnt1 == 0) {
      score = 7f
      ty = "无"
    } else if (cnt1 <= 2) {
      score = 12f
      ty = "≤2"
    } else {
      score = 15f
      ty = "＞2"
    }
    (id, cid, kind, DimScoreV2.newsEventMap.get(kind), project, DimScoreV2.newsEventMap.get(project), ty,
      score, total, extraScore)
  }

}
